@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package remote
@@ -14,6 +16,8 @@ import (
 	version "github.com/hashicorp/go-version"
 	"github.com/hashicorp/terraform-svchost/disco"
 	"github.com/opentofu/opentofu/internal/backend"
+	"github.com/opentofu/opentofu/internal/encryption"
+	"github.com/opentofu/opentofu/internal/states/statemgr"
 	"github.com/opentofu/opentofu/internal/tfdiags"
 	tfversion "github.com/opentofu/opentofu/version"
 	"github.com/zclconf/go-cty/cty"
@@ -22,8 +26,8 @@ import (
 )
 
 func TestRemote(t *testing.T) {
-	var _ backend.Enhanced = New(nil)
-	var _ backend.CLI = New(nil)
+	var _ backend.Enhanced = New(nil, encryption.StateEncryptionDisabled())
+	var _ backend.CLI = New(nil, encryption.StateEncryptionDisabled())
 }
 
 func TestRemote_backendDefault(t *testing.T) {
@@ -150,7 +154,7 @@ func TestRemote_config(t *testing.T) {
 
 	for name, tc := range cases {
 		s := testServer(t)
-		b := New(testDisco(s))
+		b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 		// Validate
 		_, valDiags := b.PrepareConfig(tc.config)
@@ -225,7 +229,7 @@ func TestRemote_versionConstraints(t *testing.T) {
 
 	for name, tc := range cases {
 		s := testServer(t)
-		b := New(testDisco(s))
+		b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 		// Set the version for this test.
 		tfversion.Prerelease = tc.prerelease
@@ -530,6 +534,46 @@ func TestRemote_StateMgr_versionCheck(t *testing.T) {
 	}
 }
 
+func TestRemote_Unlock_ignoreVersion(t *testing.T) {
+	b, bCleanup := testBackendDefault(t)
+	defer bCleanup()
+
+	// this is set by the unlock command
+	b.IgnoreVersionConflict()
+
+	v111 := version.Must(version.NewSemver("1.1.1"))
+
+	// Save original local version state and restore afterwards
+	p := tfversion.Prerelease
+	v := tfversion.Version
+	s := tfversion.SemVer
+	defer func() {
+		tfversion.Prerelease = p
+		tfversion.Version = v
+		tfversion.SemVer = s
+	}()
+
+	// For this test, the local Terraform version is set to 1.1.1
+	tfversion.Prerelease = ""
+	tfversion.Version = v111.String()
+	tfversion.SemVer = v111
+
+	state, err := b.StateMgr(backend.DefaultStateName)
+	if err != nil {
+		t.Fatalf("error: %v", err)
+	}
+
+	lockID, err := state.Lock(statemgr.NewLockInfo())
+	if err != nil {
+		t.Fatalf("error: %v", err)
+	}
+
+	// this should succeed since the version conflict is ignored
+	if err = state.Unlock(lockID); err != nil {
+		t.Fatalf("error: %v", err)
+	}
+}
+
 func TestRemote_StateMgr_versionCheckLatest(t *testing.T) {
 	b, bCleanup := testBackendDefault(t)
 	defer bCleanup()
@@ -740,7 +784,7 @@ func TestRemote_VerifyWorkspaceTerraformVersion_ignoreFlagSet(t *testing.T) {
 
 func TestRemote_ServiceDiscoveryAliases(t *testing.T) {
 	s := testServer(t)
-	b := New(testDisco(s))
+	b := New(testDisco(s), encryption.StateEncryptionDisabled())
 
 	diag := b.Configure(cty.ObjectVal(map[string]cty.Value{
 		"hostname":     cty.StringVal(mockedBackendHost),
