@@ -24,6 +24,7 @@ type RefreshCommand struct {
 
 func (c *RefreshCommand) Run(rawArgs []string) int {
 	var diags tfdiags.Diagnostics
+	ctx := c.CommandContext()
 
 	// Parse and apply global view arguments
 	common, rawArgs := arguments.ParseView(rawArgs)
@@ -103,7 +104,7 @@ func (c *RefreshCommand) Run(rawArgs []string) int {
 	diags = nil
 
 	// Perform the operation
-	op, diags := c.RunOperation(be, opReq)
+	op, diags := c.RunOperation(ctx, be, opReq)
 	view.Diagnostics(diags)
 	if diags.HasErrors() {
 		return 1
@@ -150,6 +151,7 @@ func (c *RefreshCommand) OperationRequest(be backend.Enhanced, view views.Refres
 	opReq.ConfigDir = "."
 	opReq.Hooks = view.Hooks()
 	opReq.Targets = args.Targets
+	opReq.Excludes = args.Excludes
 	opReq.Type = backend.OperationTypeRefresh
 	opReq.View = view.Operation()
 
@@ -165,7 +167,7 @@ func (c *RefreshCommand) OperationRequest(be backend.Enhanced, view views.Refres
 
 func (c *RefreshCommand) GatherVariables(args *arguments.Vars) {
 	// FIXME the arguments package currently trivially gathers variable related
-	// arguments in a heterogenous slice, in order to minimize the number of
+	// arguments in a heterogeneous slice, in order to minimize the number of
 	// code paths gathering variables during the transition to this structure.
 	// Once all commands that gather variables have been converted to this
 	// structure, we could move the variable gathering code to the arguments
@@ -193,32 +195,46 @@ Usage: tofu [global options] refresh [options]
 
 Options:
 
-  -compact-warnings   If OpenTofu produces any warnings that are not
-                      accompanied by errors, show them in a more compact form
-                      that includes only the summary messages.
+  -compact-warnings      If OpenTofu produces any warnings that are not
+                         accompanied by errors, show them in a more compact form
+                         that includes only the summary messages.
 
-  -input=true         Ask for input for variables if not directly set.
+  -consolidate-warnings  If OpenTofu produces any warnings, no consolodation
+                         will be performed. All locations, for all warnings
+                         will be listed. Enabled by default.
 
-  -lock=false         Don't hold a state lock during the operation. This is
-                      dangerous if others might concurrently run commands
-                      against the same workspace.
+  -consolidate-errors    If OpenTofu produces any errors, no consolodation
+                         will be performed. All locations, for all errors
+                         will be listed. Disabled by default
 
-  -lock-timeout=0s    Duration to retry a state lock.
+  -exclude=resource      Resource to exclude. Operation will be limited to all
+                         resources that are not excluded or dependent on excluded
+                         resources. This flag can be used multiple times. Cannot
+                         be used alongside the -target flag.
 
-  -no-color           If specified, output won't contain any color.
+  -input=true            Ask for input for variables if not directly set.
 
-  -parallelism=n      Limit the number of concurrent operations. Defaults to 10.
+  -lock=false            Don't hold a state lock during the operation. This is
+                         dangerous if others might concurrently run commands
+                         against the same workspace.
 
-  -target=resource    Resource to target. Operation will be limited to this
-                      resource and its dependencies. This flag can be used
-                      multiple times.
+  -lock-timeout=0s       Duration to retry a state lock.
 
-  -var 'foo=bar'      Set a variable in the OpenTofu configuration. This
-                      flag can be set multiple times.
+  -no-color              If specified, output won't contain any color.
 
-  -var-file=foo       Set variables in the OpenTofu configuration from
-                      a file. If "terraform.tfvars" or any ".auto.tfvars"
-                      files are present, they will be automatically loaded.
+  -parallelism=n         Limit the number of concurrent operations. Defaults to 10.
+
+  -target=resource       Resource to target. Operation will be limited to this
+                         resource and its dependencies. This flag can be used
+                         multiple times.  Cannot be used alongside the -exclude
+                         flag.
+
+  -var 'foo=bar'         Set a variable in the OpenTofu configuration. This
+                         flag can be set multiple times.
+
+  -var-file=foo          Set variables in the OpenTofu configuration from
+                         a file. If "terraform.tfvars" or any ".auto.tfvars"
+                         files are present, they will be automatically loaded.
 
   -state, state-out, and -backup are legacy options supported for the local
   backend only. For more information, see the local backend's documentation.
