@@ -31,10 +31,16 @@ type StateShowCommand struct {
 }
 
 func (c *StateShowCommand) Run(args []string) int {
+	ctx := c.CommandContext()
+
 	args = c.Meta.process(args)
 	cmdFlags := c.Meta.defaultFlagSet("state show")
 	c.Meta.varFlagSet(cmdFlags)
 	cmdFlags.StringVar(&c.Meta.statePath, "state", "", "path")
+
+	showSensitive := false
+	cmdFlags.BoolVar(&showSensitive, "show-sensitive", false, "displays sensitive values")
+
 	if err := cmdFlags.Parse(args); err != nil {
 		c.Streams.Eprintf("Error parsing command-line flags: %s\n", err.Error())
 		return 1
@@ -108,7 +114,7 @@ func (c *StateShowCommand) Run(args []string) int {
 	}
 
 	// Get the context (required to get the schemas)
-	lr, _, ctxDiags := local.LocalRun(opReq)
+	lr, _, ctxDiags := local.LocalRun(ctx, opReq)
 	if ctxDiags.HasErrors() {
 		c.View.Diagnostics(ctxDiags)
 		return 1
@@ -168,6 +174,7 @@ func (c *StateShowCommand) Run(args []string) int {
 		addr.Resource,
 		is.Current,
 		absPc,
+		addrs.NoKey,
 	)
 
 	root, outputs, err := jsonstate.MarshalForRenderer(statefile.New(singleInstance, "", 0), schemas)
@@ -187,6 +194,7 @@ func (c *StateShowCommand) Run(args []string) int {
 		Streams:             c.Streams,
 		Colorize:            c.Colorize(),
 		RunningInAutomation: c.RunningInAutomation,
+		ShowSensitive:       showSensitive,
 	}
 
 	renderer.RenderHumanState(jstate)
@@ -209,6 +217,8 @@ Options:
                       up OpenTofu-managed resources. By default it will
                       use the state "terraform.tfstate" if it exists.
 
+  -show-sensitive     If specified, sensitive values will be displayed.
+
   -var 'foo=bar'      Set a value for one of the input variables in the root
                       module of the configuration. Use this option more than
                       once to set more than one variable.
