@@ -60,6 +60,18 @@ Usage: tofu [global options] test [options]
 
 Options:
 
+  -compact-warnings     If OpenTofu produces any warnings that are not
+                        accompanied by errors, show them in a more compact
+                        form that includes only the summary messages.
+
+  -consolidate-warnings If OpenTofu produces any warnings, no consolodation
+                        will be performed. All locations, for all warnings
+                        will be listed. Enabled by default.
+
+  -consolidate-errors   If OpenTofu produces any errors, no consolodation
+                        will be performed. All locations, for all errors
+                        will be listed. Disabled by default
+
   -filter=testfile      If specified, OpenTofu will only execute the test files
                         specified by this flag. You can use this option multiple
                         times to execute more than one test file.
@@ -103,6 +115,7 @@ func (c *TestCommand) Synopsis() string {
 
 func (c *TestCommand) Run(rawArgs []string) int {
 	var diags tfdiags.Diagnostics
+	ctx := c.CommandContext()
 
 	common, rawArgs := arguments.ParseView(rawArgs)
 	c.View.Configure(common)
@@ -240,9 +253,9 @@ func (c *TestCommand) Run(rawArgs []string) int {
 	// if we're halfway through a test. We'll print details explaining what was
 	// stopped so the user can do their best to recover from it.
 
-	runningCtx, done := context.WithCancel(context.Background())
+	runningCtx, done := context.WithCancel(context.WithoutCancel(ctx))
 	stopCtx, stop := context.WithCancel(runningCtx)
-	cancelCtx, cancel := context.WithCancel(context.Background())
+	cancelCtx, cancel := context.WithCancel(context.WithoutCancel(ctx))
 
 	runner := &TestSuiteRunner{
 		command: c,
@@ -274,7 +287,7 @@ func (c *TestCommand) Run(rawArgs []string) int {
 		defer stop()
 		defer cancel()
 
-		runner.Start(variables)
+		runner.Start(ctx)
 	}()
 
 	// Wait for the operation to complete, or for an interrupt to occur.
@@ -355,7 +368,7 @@ type TestSuiteRunner struct {
 	Verbose bool
 }
 
-func (runner *TestSuiteRunner) Start(globals map[string]backend.UnparsedVariableValue) {
+func (runner *TestSuiteRunner) Start(ctx context.Context) {
 	var files []string
 	for name := range runner.Suite.Files {
 		files = append(files, name)
@@ -380,8 +393,8 @@ func (runner *TestSuiteRunner) Start(globals map[string]backend.UnparsedVariable
 			},
 		}
 
-		fileRunner.ExecuteTestFile(file)
-		fileRunner.Cleanup(file)
+		fileRunner.ExecuteTestFile(ctx, file)
+		fileRunner.Cleanup(ctx, file)
 		runner.Suite.Status = runner.Suite.Status.Merge(file.Status)
 	}
 }
@@ -397,7 +410,7 @@ type TestFileState struct {
 	State *states.State
 }
 
-func (runner *TestFileRunner) ExecuteTestFile(file *moduletest.File) {
+func (runner *TestFileRunner) ExecuteTestFile(ctx context.Context, file *moduletest.File) {
 	log.Printf("[TRACE] TestFileRunner: executing test file %s", file.Name)
 
 	file.Status = file.Status.Merge(moduletest.Pass)
@@ -455,7 +468,7 @@ func (runner *TestFileRunner) ExecuteTestFile(file *moduletest.File) {
 			}
 		}
 
-		state, updatedState := runner.ExecuteTestRun(run, file, runner.States[key].State, config)
+		state, updatedState := runner.ExecuteTestRun(ctx, run, file, runner.States[key].State, config)
 		if updatedState {
 			var err error
 
@@ -494,7 +507,8 @@ func (runner *TestFileRunner) ExecuteTestFile(file *moduletest.File) {
 	}
 }
 
-func (runner *TestFileRunner) ExecuteTestRun(run *moduletest.Run, file *moduletest.File, state *states.State, config *configs.Config) (*states.State, bool) {
+//nolint:funlen // Historical function predates our complexity rules
+func (runner *TestFileRunner) ExecuteTestRun(ctx context.Context, run *moduletest.Run, file *moduletest.File, state *states.State, config *configs.Config) (*states.State, bool) {
 	log.Printf("[TRACE] TestFileRunner: executing run block %s/%s", file.Name, run.Name)
 
 	if runner.Suite.Cancelled {
@@ -531,17 +545,18 @@ func (runner *TestFileRunner) ExecuteTestRun(run *moduletest.Run, file *modulete
 		return state, false
 	}
 
-	validateDiags := runner.validate(config, run, file)
+	validateDiags := runner.validate(ctx, config, run, file)
 	run.Diagnostics = run.Diagnostics.Append(validateDiags)
 	if validateDiags.HasErrors() {
 		run.Status = moduletest.Error
 		return state, false
 	}
 
-	planCtx, plan, planDiags := runner.plan(config, state, run, file)
+	planCtx, plan, planDiags := runner.plan(ctx, config, state, run, file)
 	if run.Config.Command == configs.PlanTestCommand {
+		expectedFailures, sourceRanges := run.BuildExpectedFailuresAndSourceMaps()
 		// Then we want to assess our conditions and diagnostics differently.
-		planDiags = run.ValidateExpectedFailures(planDiags)
+		planDiags = run.ValidateExpectedFailures(expectedFailures, sourceRanges, planDiags)
 		run.Diagnostics = run.Diagnostics.Append(planDiags)
 		if planDiags.HasErrors() {
 			run.Status = moduletest.Error
@@ -586,6 +601,10 @@ func (runner *TestFileRunner) ExecuteTestRun(run *moduletest.Run, file *modulete
 		return state, false
 	}
 
+	expectedFailures, sourceRanges := run.BuildExpectedFailuresAndSourceMaps()
+
+	planDiags = checkProblematicPlanErrors(expectedFailures, planDiags)
+
 	// Otherwise any error during the planning prevents our apply from
 	// continuing which is an error.
 	run.Diagnostics = run.Diagnostics.Append(planDiags)
@@ -608,10 +627,10 @@ func (runner *TestFileRunner) ExecuteTestRun(run *moduletest.Run, file *modulete
 	}
 	run.Diagnostics = filteredDiags
 
-	applyCtx, updated, applyDiags := runner.apply(plan, state, config, run, file)
+	applyCtx, updated, applyDiags := runner.apply(ctx, plan, state, config, run, file)
 
 	// Remove expected diagnostics, and add diagnostics in case anything that should have failed didn't.
-	applyDiags = run.ValidateExpectedFailures(applyDiags)
+	applyDiags = run.ValidateExpectedFailures(expectedFailures, sourceRanges, applyDiags)
 
 	run.Diagnostics = run.Diagnostics.Append(applyDiags)
 	if applyDiags.HasErrors() {
@@ -659,7 +678,7 @@ func (runner *TestFileRunner) ExecuteTestRun(run *moduletest.Run, file *modulete
 	return updated, true
 }
 
-func (runner *TestFileRunner) validate(config *configs.Config, run *moduletest.Run, file *moduletest.File) tfdiags.Diagnostics {
+func (runner *TestFileRunner) validate(ctx context.Context, config *configs.Config, run *moduletest.Run, file *moduletest.File) tfdiags.Diagnostics {
 	log.Printf("[TRACE] TestFileRunner: called validate for %s/%s", file.Name, run.Name)
 
 	var diags tfdiags.Diagnostics
@@ -670,7 +689,7 @@ func (runner *TestFileRunner) validate(config *configs.Config, run *moduletest.R
 		return diags
 	}
 
-	runningCtx, done := context.WithCancel(context.Background())
+	runningCtx, done := context.WithCancel(context.WithoutCancel(ctx))
 
 	var validateDiags tfdiags.Diagnostics
 	panicHandler := logging.PanicHandlerWithTraceFn()
@@ -679,7 +698,7 @@ func (runner *TestFileRunner) validate(config *configs.Config, run *moduletest.R
 		defer done()
 
 		log.Printf("[DEBUG] TestFileRunner: starting validate for %s/%s", file.Name, run.Name)
-		validateDiags = tfCtx.Validate(config)
+		validateDiags = tfCtx.Validate(ctx, config)
 		log.Printf("[DEBUG] TestFileRunner: completed validate for  %s/%s", file.Name, run.Name)
 	}()
 	waitDiags, cancelled := runner.wait(tfCtx, runningCtx, run, file, nil)
@@ -694,8 +713,7 @@ func (runner *TestFileRunner) validate(config *configs.Config, run *moduletest.R
 	return diags
 }
 
-func (runner *TestFileRunner) destroy(config *configs.Config, state *states.State, run *moduletest.Run, file *moduletest.File) (*states.State, tfdiags.Diagnostics) {
-
+func (runner *TestFileRunner) destroy(ctx context.Context, config *configs.Config, state *states.State, run *moduletest.Run, file *moduletest.File) (*states.State, tfdiags.Diagnostics) {
 	log.Printf("[TRACE] TestFileRunner: called destroy for %s/%s", file.Name, run.Name)
 
 	if state.Empty() {
@@ -726,7 +744,7 @@ func (runner *TestFileRunner) destroy(config *configs.Config, state *states.Stat
 		return state, diags
 	}
 
-	runningCtx, done := context.WithCancel(context.Background())
+	runningCtx, done := context.WithCancel(context.WithoutCancel(ctx))
 
 	var plan *plans.Plan
 	var planDiags tfdiags.Diagnostics
@@ -736,7 +754,7 @@ func (runner *TestFileRunner) destroy(config *configs.Config, state *states.Stat
 		defer done()
 
 		log.Printf("[DEBUG] TestFileRunner: starting destroy plan for %s/%s", file.Name, run.Name)
-		plan, planDiags = tfCtx.Plan(config, state, planOpts)
+		plan, planDiags = tfCtx.Plan(ctx, config, state, planOpts)
 		log.Printf("[DEBUG] TestFileRunner: completed destroy plan for %s/%s", file.Name, run.Name)
 	}()
 	waitDiags, cancelled := runner.wait(tfCtx, runningCtx, run, file, nil)
@@ -752,12 +770,12 @@ func (runner *TestFileRunner) destroy(config *configs.Config, state *states.Stat
 		return state, diags
 	}
 
-	_, updated, applyDiags := runner.apply(plan, state, config, run, file)
+	_, updated, applyDiags := runner.apply(ctx, plan, state, config, run, file)
 	diags = diags.Append(applyDiags)
 	return updated, diags
 }
 
-func (runner *TestFileRunner) plan(config *configs.Config, state *states.State, run *moduletest.Run, file *moduletest.File) (*tofu.Context, *plans.Plan, tfdiags.Diagnostics) {
+func (runner *TestFileRunner) plan(ctx context.Context, config *configs.Config, state *states.State, run *moduletest.Run, file *moduletest.File) (*tofu.Context, *plans.Plan, tfdiags.Diagnostics) {
 	log.Printf("[TRACE] TestFileRunner: called plan for %s/%s", file.Name, run.Name)
 
 	var diags tfdiags.Diagnostics
@@ -803,7 +821,7 @@ func (runner *TestFileRunner) plan(config *configs.Config, state *states.State,
 		return nil, nil, diags
 	}
 
-	runningCtx, done := context.WithCancel(context.Background())
+	runningCtx, done := context.WithCancel(context.WithoutCancel(ctx))
 
 	var plan *plans.Plan
 	var planDiags tfdiags.Diagnostics
@@ -813,7 +831,7 @@ func (runner *TestFileRunner) plan(config *configs.Config, state *states.State,
 		defer done()
 
 		log.Printf("[DEBUG] TestFileRunner: starting plan for %s/%s", file.Name, run.Name)
-		plan, planDiags = tfCtx.Plan(config, state, planOpts)
+		plan, planDiags = tfCtx.Plan(ctx, config, state, planOpts)
 		log.Printf("[DEBUG] TestFileRunner: completed plan for %s/%s", file.Name, run.Name)
 	}()
 	waitDiags, cancelled := runner.wait(tfCtx, runningCtx, run, file, nil)
@@ -828,7 +846,7 @@ func (runner *TestFileRunner) plan(config *configs.Config, state *states.State,
 	return tfCtx, plan, diags
 }
 
-func (runner *TestFileRunner) apply(plan *plans.Plan, state *states.State, config *configs.Config, run *moduletest.Run, file *moduletest.File) (*tofu.Context, *states.State, tfdiags.Diagnostics) {
+func (runner *TestFileRunner) apply(ctx context.Context, plan *plans.Plan, state *states.State, config *configs.Config, run *moduletest.Run, file *moduletest.File) (*tofu.Context, *states.State, tfdiags.Diagnostics) {
 	log.Printf("[TRACE] TestFileRunner: called apply for %s/%s", file.Name, run.Name)
 
 	var diags tfdiags.Diagnostics
@@ -858,7 +876,7 @@ func (runner *TestFileRunner) apply(plan *plans.Plan, state *states.State, confi
 		return nil, state, diags
 	}
 
-	runningCtx, done := context.WithCancel(context.Background())
+	runningCtx, done := context.WithCancel(context.WithoutCancel(ctx))
 
 	var updated *states.State
 	var applyDiags tfdiags.Diagnostics
@@ -868,7 +886,7 @@ func (runner *TestFileRunner) apply(plan *plans.Plan, state *states.State, confi
 		defer panicHandler()
 		defer done()
 		log.Printf("[DEBUG] TestFileRunner: starting apply for %s/%s", file.Name, run.Name)
-		updated, applyDiags = tfCtx.Apply(plan, config)
+		updated, applyDiags = tfCtx.Apply(ctx, plan, config)
 		log.Printf("[DEBUG] TestFileRunner: completed apply for %s/%s", file.Name, run.Name)
 	}()
 	waitDiags, cancelled := runner.wait(tfCtx, runningCtx, run, file, created)
@@ -951,7 +969,7 @@ func (runner *TestFileRunner) wait(ctx *tofu.Context, runningCtx context.Context
 	return diags, cancelled
 }
 
-func (runner *TestFileRunner) Cleanup(file *moduletest.File) {
+func (runner *TestFileRunner) Cleanup(ctx context.Context, file *moduletest.File) {
 	log.Printf("[TRACE] TestStateManager: cleaning up state for %s", file.Name)
 
 	if runner.Suite.Cancelled {
@@ -1023,7 +1041,7 @@ func (runner *TestFileRunner) Cleanup(file *moduletest.File) {
 		updated := state.State
 		if !diags.HasErrors() {
 			var destroyDiags tfdiags.Diagnostics
-			updated, destroyDiags = runner.destroy(runConfig, state.State, state.Run, file)
+			updated, destroyDiags = runner.destroy(ctx, runConfig, state.State, state.Run, file)
 			diags = diags.Append(destroyDiags)
 		}
 		runner.Suite.View.DestroySummary(diags, state.Run, file, updated)
@@ -1284,6 +1302,29 @@ func (runner *TestFileRunner) prepareInputVariablesForAssertions(config *configs
 	}, diags
 }
 
+// checkProblematicPlanErrors checks for plan errors that are also "expected" by the tests. In some cases we expect an error, however,
+// what causes the error might not be what we expected. So we try to warn about that here.
+func checkProblematicPlanErrors(expectedFailures addrs.Map[addrs.Referenceable, bool], planDiags tfdiags.Diagnostics) tfdiags.Diagnostics {
+	for _, diag := range planDiags {
+		rule, ok := addrs.DiagnosticOriginatesFromCheckRule(diag)
+		if !ok {
+			continue
+		}
+		if rule.Container.CheckableKind() != addrs.CheckableInputVariable {
+			continue
+		}
+
+		addr, ok := rule.Container.(addrs.AbsInputVariableInstance)
+		if ok && expectedFailures.Has(addr.Variable) {
+			planDiags = planDiags.Append(tfdiags.Sourceless(
+				tfdiags.Warning,
+				"Invalid Variable in test file",
+				fmt.Sprintf("Variable %s, has an invalid value within the test. Although this was an expected failure, it has meant the apply stage was unable to run so the overall test will fail.", rule.Container.String())))
+		}
+	}
+	return planDiags
+}
+
 // simulateStateSerialization takes a state, serializes it, deserializes it
 // and then returns. This is useful for state writing side effects without
 // actually writing a state file.
