@@ -212,7 +212,7 @@ func (c *Config) EntersNewPackage() bool {
 
 // VerifyDependencySelections checks whether the given locked dependencies
 // are acceptable for all of the version constraints reported in the
-// configuration tree represented by the reciever.
+// configuration tree represented by the receiver.
 //
 // This function will errors only if any of the locked dependencies are out of
 // range for corresponding constraints in the configuration. If there are
@@ -966,15 +966,16 @@ func (c *Config) transformProviderConfigsForTest(run *TestRun, file *TestFile) (
 			}
 
 			next[ref.InChild.String()] = &Provider{
-				Name:          ref.InChild.Name,
-				NameRange:     ref.InChild.NameRange,
-				Alias:         ref.InChild.Alias,
-				AliasRange:    ref.InChild.AliasRange,
-				Version:       testProvider.Version,
-				Config:        testProvider.Config,
-				DeclRange:     testProvider.DeclRange,
-				IsMocked:      testProvider.IsMocked,
-				MockResources: testProvider.MockResources,
+				Name:              ref.InChild.Name,
+				NameRange:         ref.InChild.NameRange,
+				Alias:             ref.InChild.Alias,
+				AliasRange:        ref.InChild.AliasRange,
+				Version:           testProvider.Version,
+				Config:            testProvider.Config,
+				DeclRange:         testProvider.DeclRange,
+				IsMocked:          testProvider.IsMocked,
+				MockResources:     testProvider.MockResources,
+				OverrideResources: testProvider.OverrideResources,
 			}
 
 		}
@@ -986,13 +987,14 @@ func (c *Config) transformProviderConfigsForTest(run *TestRun, file *TestFile) (
 		}
 		for _, mp := range file.MockProviders {
 			next[mp.moduleUniqueKey()] = &Provider{
-				Name:          mp.Name,
-				NameRange:     mp.NameRange,
-				Alias:         mp.Alias,
-				AliasRange:    mp.AliasRange,
-				DeclRange:     mp.DeclRange,
-				IsMocked:      true,
-				MockResources: mp.MockResources,
+				Name:              mp.Name,
+				NameRange:         mp.NameRange,
+				Alias:             mp.Alias,
+				AliasRange:        mp.AliasRange,
+				DeclRange:         mp.DeclRange,
+				IsMocked:          true,
+				MockResources:     mp.MockResources,
+				OverrideResources: mp.OverrideResources,
 			}
 		}
 	}
