@@ -1,4 +1,6 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright (c) The OpenTofu Authors
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) 2023 HashiCorp, Inc.
 // SPDX-License-Identifier: MPL-2.0
 
 package configs
@@ -7,6 +9,7 @@ import (
 	"bytes"
 	"fmt"
 	"os"
+	"path/filepath"
 	"strings"
 	"testing"
 
@@ -321,7 +324,7 @@ func TestConfigProviderRequirementsByModule(t *testing.T) {
 			"kinder": {
 				Name:       "kinder",
 				SourceAddr: addrs.ModuleSourceLocal("./child"),
-				SourceDir:  "testdata/provider-reqs/child",
+				SourceDir:  filepath.FromSlash("testdata/provider-reqs/child"),
 				Requirements: getproviders.Requirements{
 					nullProvider:       getproviders.MustParseVersionConstraints("= 2.0.1"),
 					happycloudProvider: nil,
@@ -330,7 +333,7 @@ func TestConfigProviderRequirementsByModule(t *testing.T) {
 					"nested": {
 						Name:       "nested",
 						SourceAddr: addrs.ModuleSourceLocal("./grandchild"),
-						SourceDir:  "testdata/provider-reqs/child/grandchild",
+						SourceDir:  filepath.FromSlash("testdata/provider-reqs/child/grandchild"),
 						Requirements: getproviders.Requirements{
 							grandchildProvider: nil,
 						},
@@ -391,7 +394,7 @@ func TestConfigProviderRequirementsByModuleInclTests(t *testing.T) {
 					"setup": {
 						Name:       "setup",
 						SourceAddr: addrs.ModuleSourceLocal("./setup"),
-						SourceDir:  "testdata/provider-reqs-with-tests/setup",
+						SourceDir:  filepath.FromSlash("testdata/provider-reqs-with-tests/setup"),
 						Requirements: getproviders.Requirements{
 							nullProvider:   getproviders.MustParseVersionConstraints("~> 2.0.0"),
 							randomProvider: getproviders.MustParseVersionConstraints("~> 1.2.0"),
