@@ -130,25 +130,9 @@ func (run *Run) GetReferences() ([]*addrs.Reference, tfdiags.Diagnostics) {
 // diagnostics were generated by custom conditions. OpenTofu adds the
 // addrs.CheckRule that generated each diagnostic to the diagnostic itself so we
 // can tell which diagnostics can be expected.
-func (run *Run) ValidateExpectedFailures(originals tfdiags.Diagnostics) tfdiags.Diagnostics {
-
-	// We're going to capture all the checkable objects that are referenced
-	// from the expected failures.
-	expectedFailures := addrs.MakeMap[addrs.Referenceable, bool]()
-	sourceRanges := addrs.MakeMap[addrs.Referenceable, tfdiags.SourceRange]()
-
-	for _, traversal := range run.Config.ExpectFailures {
-		// Ignore the diagnostics returned from the reference parsing, these
-		// references will have been checked earlier in the process by the
-		// validate stage so we don't need to do that again here.
-		reference, _ := addrs.ParseRefFromTestingScope(traversal)
-		expectedFailures.Put(reference.Subject, false)
-		sourceRanges.Put(reference.Subject, reference.SourceRange)
-	}
-
+func (run *Run) ValidateExpectedFailures(expectedFailures addrs.Map[addrs.Referenceable, bool], sourceRanges addrs.Map[addrs.Referenceable, tfdiags.SourceRange], originals tfdiags.Diagnostics) tfdiags.Diagnostics { //nolint: funlen,gocognit,cyclop // legacy code causes this to trigger, we actually reduced the statements
 	var diags tfdiags.Diagnostics
 	for _, diag := range originals {
-
 		if rule, ok := addrs.DiagnosticOriginatesFromCheckRule(diag); ok {
 			switch rule.Container.CheckableKind() {
 			case addrs.CheckableOutputValue:
@@ -340,3 +324,20 @@ func (run *Run) ValidateExpectedFailures(originals tfdiags.Diagnostics) tfdiags.
 
 	return diags
 }
+
+// BuildExpectedFailuresAndSourceMaps captures all the checkable objects that are referenced
+// from the expected failures.
+func (run *Run) BuildExpectedFailuresAndSourceMaps() (addrs.Map[addrs.Referenceable, bool], addrs.Map[addrs.Referenceable, tfdiags.SourceRange]) {
+	expectedFailures := addrs.MakeMap[addrs.Referenceable, bool]()
+	sourceRanges := addrs.MakeMap[addrs.Referenceable, tfdiags.SourceRange]()
+
+	for _, traversal := range run.Config.ExpectFailures {
+		// Ignore the diagnostics returned from the reference parsing, these
+		// references will have been checked earlier in the process by the
+		// validate stage so we don't need to do that again here.
+		reference, _ := addrs.ParseRefFromTestingScope(traversal)
+		expectedFailures.Put(reference.Subject, false)
+		sourceRanges.Put(reference.Subject, reference.SourceRange)
+	}
+	return expectedFailures, sourceRanges
+}
